@@ -70,7 +70,6 @@ _gnutls_get_public_rsa_params (gnutls_session_t session,
   int ret;
   cert_auth_info_t info;
   gnutls_pcert_st peer_cert;
-  unsigned int i;
 
   /* normal non export case */
 
@@ -95,34 +94,6 @@ _gnutls_get_public_rsa_params (gnutls_session_t session,
 
   gnutls_pk_params_init(params);
 
-  /* EXPORT case: */
-  if (_gnutls_cipher_suite_get_kx_algo
-      (session->security_parameters.cipher_suite) ==
-      GNUTLS_KX_RSA_EXPORT &&
-      _gnutls_pubkey_is_over_rsa_512(peer_cert.pubkey) == 0)
-    {
-      if (session->key.rsa[0] == NULL || session->key.rsa[1] == NULL)
-        {
-          gnutls_assert ();
-          ret = GNUTLS_E_INTERNAL_ERROR;
-          goto cleanup;
-        }
-
-      for (i = 0; i < RSA_PUBLIC_PARAMS; i++)
-        {
-          params->params[i] = _gnutls_mpi_copy (session->key.rsa[i]);
-          if (params->params[i] != NULL)
-            params->params_nr++;
-          else
-            {
-              ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-              goto cleanup;
-            }
-        }
-      
-      goto done;
-    }
-
   ret = _gnutls_pubkey_get_mpis(peer_cert.pubkey, params);
   if (ret < 0)
     {
@@ -130,13 +101,9 @@ _gnutls_get_public_rsa_params (gnutls_session_t session,
       goto cleanup2;
     }
 
-done:
   gnutls_pcert_deinit (&peer_cert);
   return 0;
   
-cleanup:
-  for (i=0;i<params->params_nr;i++)
-    _gnutls_mpi_release(params->params[i]);
 cleanup2:
   gnutls_pcert_deinit (&peer_cert);
 
@@ -153,7 +120,7 @@ proc_rsa_client_kx (gnutls_session_t session, uint8_t * data,
   int randomize_key = 0;
   ssize_t data_size = _data_size;
 
-  if (gnutls_protocol_get_version (session) == GNUTLS_SSL3)
+  if (get_num_version (session) == GNUTLS_SSL3)
     {
       /* SSL 3.0 
        */
@@ -258,7 +225,6 @@ _gnutls_gen_rsa_client_kx (gnutls_session_t session, gnutls_buffer_st* data)
   gnutls_datum_t sdata;         /* data to send */
   gnutls_pk_params_st params;
   int ret;
-  gnutls_protocol_t ver;
 
   if (auth == NULL)
     {
@@ -286,12 +252,10 @@ _gnutls_gen_rsa_client_kx (gnutls_session_t session, gnutls_buffer_st* data)
       return ret;
     }
 
-  ver = _gnutls_get_adv_version (session);
-
   if (session->internals.rsa_pms_version[0] == 0)
     {
-      session->key.key.data[0] = _gnutls_version_get_major (ver);
-      session->key.key.data[1] = _gnutls_version_get_minor (ver);
+      session->key.key.data[0] = _gnutls_get_adv_version_major(session);
+      session->key.key.data[1] = _gnutls_get_adv_version_minor(session);
     }
   else
     {                           /* use the version provided */
@@ -318,7 +282,7 @@ _gnutls_gen_rsa_client_kx (gnutls_session_t session, gnutls_buffer_st* data)
     return gnutls_assert_val(ret);
 
 
-  if (gnutls_protocol_get_version (session) == GNUTLS_SSL3)
+  if (get_num_version (session) == GNUTLS_SSL3)
     {
       /* SSL 3.0 */
       _gnutls_buffer_replace_data( data, &sdata);
