@@ -97,8 +97,8 @@ digest_ticket (const gnutls_datum_t * key, struct ticket *ticket,
   uint16_t length16;
   int ret;
 
-  ret = _gnutls_mac_init (&digest_hd, GNUTLS_MAC_SHA256, key->data,
-                           key->size);
+  ret = _gnutls_mac_init (&digest_hd, mac_to_entry(GNUTLS_MAC_SHA256), 
+                          key->data, key->size);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -147,7 +147,8 @@ decrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
   IV.data = ticket->IV;
   IV.size = IV_SIZE;
   ret =
-    _gnutls_cipher_init (&cipher_hd, GNUTLS_CIPHER_AES_128_CBC, &key, &IV, 0);
+    _gnutls_cipher_init (&cipher_hd, cipher_to_entry(GNUTLS_CIPHER_AES_128_CBC), 
+                         &key, &IV, 0);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -222,7 +223,8 @@ encrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
   IV.data = priv->session_ticket_IV;
   IV.size = IV_SIZE;
   ret =
-    _gnutls_cipher_init (&cipher_hd, GNUTLS_CIPHER_AES_128_CBC, &key, &IV, 1);
+    _gnutls_cipher_init (&cipher_hd, cipher_to_entry(GNUTLS_CIPHER_AES_128_CBC),
+                         &key, &IV, 1);
   if (ret < 0)
     {
       gnutls_assert ();
