@@ -38,6 +38,7 @@
 #include "gnutls_kx.h"
 #include "gnutls_record.h"
 #include "gnutls_constate.h"
+#include "gnutls_mbuffers.h"
 #include <gnutls_state.h>
 #include <random.h>
 
@@ -91,10 +92,11 @@ is_read_comp_null (record_parameters_st * record_params)
  * 
  */
 int
-_gnutls_encrypt (gnutls_session_t session, const uint8_t * headers,
-                 size_t headers_size, const uint8_t * data,
-                 size_t data_size, size_t target_size, uint8_t * ciphertext,
-                 size_t ciphertext_size, content_type_t type, 
+_gnutls_encrypt (gnutls_session_t session, 
+                 const uint8_t * data, size_t data_size, 
+                 size_t target_size, 
+                 mbuffer_st* bufel,
+                 content_type_t type, 
                  record_parameters_st * params)
 {
   gnutls_datum_t comp;
@@ -113,7 +115,7 @@ _gnutls_encrypt (gnutls_session_t session, const uint8_t * headers,
        */
       free_comp = 1;
       
-      comp.size = ciphertext_size - headers_size;
+      comp.size = _mbuffer_get_udata_size(bufel);
       comp.data = gnutls_malloc(comp.size);
       if (comp.data == NULL)
         return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
@@ -130,12 +132,12 @@ _gnutls_encrypt (gnutls_session_t session, const uint8_t * headers,
     }
 
   if (session->security_parameters.new_record_padding != 0)
-    ret = compressed_to_ciphertext_new (session, &ciphertext[headers_size],
-                                        ciphertext_size - headers_size,
+    ret = compressed_to_ciphertext_new (session, _mbuffer_get_udata_ptr(bufel),
+                                        _mbuffer_get_udata_size(bufel),
                                         &comp, target_size, type, params);
   else
-    ret = compressed_to_ciphertext (session, &ciphertext[headers_size],
-                                    ciphertext_size - headers_size,
+    ret = compressed_to_ciphertext (session, _mbuffer_get_udata_ptr(bufel),
+                                    _mbuffer_get_udata_size(bufel),
                                     &comp, target_size, type, params);
 
   if (free_comp)
@@ -144,15 +146,15 @@ _gnutls_encrypt (gnutls_session_t session, const uint8_t * headers,
   if (ret < 0)
     return gnutls_assert_val(ret);
 
-  /* copy the headers */
-  memcpy (ciphertext, headers, headers_size);
-  
   if(IS_DTLS(session))
-    _gnutls_write_uint16 (ret, &ciphertext[11]);
+    _gnutls_write_uint16 (ret, ((uint8_t*)_mbuffer_get_uhead_ptr(bufel))+11);
   else
-    _gnutls_write_uint16 (ret, &ciphertext[3]);
+    _gnutls_write_uint16 (ret, ((uint8_t*)_mbuffer_get_uhead_ptr(bufel))+3);
+    
+  _mbuffer_set_udata_size(bufel, ret);
+  _mbuffer_set_uhead_size(bufel, 0);
 
-  return ret + headers_size;
+  return _mbuffer_get_udata_size(bufel);
 }
 
 /* Decrypts the given data.
@@ -226,7 +228,9 @@ leave:
 
 
 inline static int
-calc_enc_length_block (gnutls_session_t session, int data_size,
+calc_enc_length_block (gnutls_session_t session, 
+                 const version_entry_st* ver,
+                 int data_size,
                  int hash_size, uint8_t * pad, 
                  unsigned auth_cipher, uint16_t blocksize)
 {
@@ -244,8 +248,7 @@ calc_enc_length_block (gnutls_session_t session, int data_size,
 
   length = data_size + hash_size + *pad;
 
-  if (_gnutls_version_has_explicit_iv
-      (session->security_parameters.version))
+  if (_gnutls_version_has_explicit_iv(ver))
     length += blocksize;    /* for the IV */
 
   return length;
@@ -271,25 +274,22 @@ calc_enc_length_stream (gnutls_session_t session, int data_size,
  */
 static inline int
 make_preamble (uint8_t * uint64_data, uint8_t type, unsigned int length,
-               gnutls_protocol_t ver, uint8_t * preamble)
+               const version_entry_st* ver, uint8_t * preamble)
 {
-  uint8_t minor, major;
   uint8_t *p = preamble;
   uint16_t c_length;
   
-  _gnutls_version_to_tls(ver, &major, &minor);
-
   c_length = _gnutls_conv_uint16 (length);
 
   memcpy (p, uint64_data, 8);
   p += 8;
   *p = type;
   p++;
-  if (ver != GNUTLS_SSL3)
+  if (ver->id != GNUTLS_SSL3)
     { /* TLS protocols */
-      *p = major;
+      *p = ver->major;
       p++;
-      *p = minor;
+      *p = ver->minor;
       p++;
     }
   memcpy (p, &c_length, 2);
@@ -310,26 +310,28 @@ compressed_to_ciphertext (gnutls_session_t session,
                           content_type_t type, 
                           record_parameters_st * params)
 {
-  uint8_t * tag_ptr = NULL;
   uint8_t pad = target_size - compressed->size;
-  int length, length_to_encrypt, ret;
+  int length, ret;
   uint8_t preamble[MAX_PREAMBLE_SIZE];
   int preamble_size;
   int tag_size = _gnutls_auth_cipher_tag_len (&params->write.cipher_state);
-  int blocksize = gnutls_cipher_get_block_size (params->cipher_algorithm);
+  int blocksize = _gnutls_cipher_get_block_size (params->cipher);
   unsigned block_algo =
-    _gnutls_cipher_is_block (params->cipher_algorithm);
+    _gnutls_cipher_is_block (params->cipher);
   uint8_t *data_ptr;
-  int ver = gnutls_protocol_get_version (session);
-  int explicit_iv = _gnutls_version_has_explicit_iv (session->security_parameters.version);
+  const version_entry_st* ver = get_version (session);
+  int explicit_iv = _gnutls_version_has_explicit_iv (ver);
   int auth_cipher = _gnutls_auth_cipher_is_aead(&params->write.cipher_state);
   uint8_t nonce[MAX_CIPHER_BLOCK_SIZE];
   unsigned iv_size;
 
-  iv_size = gnutls_cipher_get_iv_size(params->cipher_algorithm);
+  if (unlikely(ver == NULL))
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+  iv_size = _gnutls_cipher_get_iv_size(params->cipher);
 
   _gnutls_hard_log("ENC[%p]: cipher: %s, MAC: %s, Epoch: %u\n",
-    session, gnutls_cipher_get_name(params->cipher_algorithm), gnutls_mac_get_name(params->mac_algorithm),
+    session, _gnutls_cipher_get_name(params->cipher), _gnutls_mac_get_name(params->mac),
     (unsigned int)params->epoch);
 
   preamble_size =
@@ -347,28 +349,24 @@ compressed_to_ciphertext (gnutls_session_t session,
       if (ret < 0)
         return gnutls_assert_val(ret);
 
-      length_to_encrypt = length =
-        calc_enc_length_block (session, compressed->size, tag_size, &pad,
+      length =
+        calc_enc_length_block (session, ver, compressed->size, tag_size, &pad,
                                auth_cipher, blocksize);
     }
   else
     {
-      length_to_encrypt = length =
+      length =
         calc_enc_length_stream (session, compressed->size, tag_size,
                                 auth_cipher);
     }
 
   if (length < 0)
-    {
-      return gnutls_assert_val(length);
-    }
+    return gnutls_assert_val(length);
 
   /* copy the encrypted data to cipher_data.
    */
   if (cipher_size < length)
-    {
-      return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-    }
+    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
   data_ptr = cipher_data;
 
@@ -383,7 +381,6 @@ compressed_to_ciphertext (gnutls_session_t session,
 
           data_ptr += blocksize;
           cipher_data += blocksize;
-          length_to_encrypt -= blocksize;
         }
       else if (auth_cipher)
         {
@@ -405,9 +402,6 @@ compressed_to_ciphertext (gnutls_session_t session,
 
           data_ptr += AEAD_EXPLICIT_DATA_SIZE;
           cipher_data += AEAD_EXPLICIT_DATA_SIZE;
-          /* In AEAD ciphers we don't encrypt the tag 
-           */
-          length_to_encrypt -= AEAD_EXPLICIT_DATA_SIZE + tag_size;
         }
       else if (iv_size > 0)
         _gnutls_auth_cipher_setiv(&params->write.cipher_state, UINT64DATA(params->write.sequence_number), 8);
@@ -422,19 +416,6 @@ compressed_to_ciphertext (gnutls_session_t session,
         _gnutls_auth_cipher_setiv(&params->write.cipher_state, UINT64DATA(params->write.sequence_number), 8);
     }
 
-  memcpy (data_ptr, compressed->data, compressed->size);
-  data_ptr += compressed->size;
-
-  if (tag_size > 0)
-    {
-      tag_ptr = data_ptr;
-      data_ptr += tag_size;
-    }
-  if (block_algo == CIPHER_BLOCK && pad > 0)
-    {
-      memset (data_ptr, pad - 1, pad);
-    }
-
   _gnutls_auth_cipher_set_mac_nonce(&params->write.cipher_state, UINT64DATA(params->write.sequence_number), 8);
 
   /* add the authenticate data */
@@ -442,13 +423,12 @@ compressed_to_ciphertext (gnutls_session_t session,
   if (ret < 0)
     return gnutls_assert_val(ret);
 
-  /* Actual encryption (inplace).
+  /* Actual encryption.
    */
   ret =
     _gnutls_auth_cipher_encrypt2_tag (&params->write.cipher_state,
-        cipher_data, length_to_encrypt, 
-        cipher_data, cipher_size,
-        tag_ptr, tag_size, compressed->size);
+        compressed->data, compressed->size, cipher_data, cipher_size,
+        pad);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -463,26 +443,28 @@ compressed_to_ciphertext_new (gnutls_session_t session,
                               content_type_t type, 
                               record_parameters_st * params)
 {
-  uint8_t * tag_ptr = NULL;
   uint16_t pad = target_size - compressed->size;
   int length, length_to_encrypt, ret;
   uint8_t preamble[MAX_PREAMBLE_SIZE];
   int preamble_size;
   int tag_size = _gnutls_auth_cipher_tag_len (&params->write.cipher_state);
-  int blocksize = gnutls_cipher_get_block_size (params->cipher_algorithm);
+  int blocksize = _gnutls_cipher_get_block_size (params->cipher);
   unsigned block_algo =
-    _gnutls_cipher_is_block (params->cipher_algorithm);
+    _gnutls_cipher_is_block (params->cipher);
   uint8_t *data_ptr;
-  int ver = gnutls_protocol_get_version (session);
-  int explicit_iv = _gnutls_version_has_explicit_iv (session->security_parameters.version);
+  const version_entry_st* ver = get_version (session);
+  int explicit_iv = _gnutls_version_has_explicit_iv (ver);
   int auth_cipher = _gnutls_auth_cipher_is_aead(&params->write.cipher_state);
   uint8_t nonce[MAX_CIPHER_BLOCK_SIZE];
   unsigned iv_size;
 
-  iv_size = gnutls_cipher_get_iv_size(params->cipher_algorithm);
+  if (unlikely(ver == NULL))
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+  iv_size = _gnutls_cipher_get_iv_size(params->cipher);
 
   _gnutls_hard_log("ENC[%p]: cipher: %s, MAC: %s, Epoch: %u\n",
-    session, gnutls_cipher_get_name(params->cipher_algorithm), gnutls_mac_get_name(params->mac_algorithm),
+    session, _gnutls_cipher_get_name(params->cipher), _gnutls_mac_get_name(params->mac),
     (unsigned int)params->epoch);
 
   /* Call _gnutls_rnd() once. Get data used for the IV
@@ -562,14 +544,12 @@ compressed_to_ciphertext_new (gnutls_session_t session,
  
   if (pad > 0)
     { 
-      DECR_LEN(cipher_size, pad);
       memset(data_ptr, 0, pad);
       data_ptr += pad;
       length_to_encrypt += pad;
       length += pad;
     }
 
-  DECR_LEN(cipher_size, compressed->size);
   memcpy (data_ptr, compressed->data, compressed->size);
   data_ptr += compressed->size;
   length_to_encrypt += compressed->size;
@@ -577,14 +557,10 @@ compressed_to_ciphertext_new (gnutls_session_t session,
 
   if (tag_size > 0)
     {
-      DECR_LEN(cipher_size, tag_size);
-      tag_ptr = data_ptr;
       data_ptr += tag_size;
       
       /* In AEAD ciphers we don't encrypt the tag 
        */
-      if (!auth_cipher)
-        length_to_encrypt += tag_size;
       length += tag_size;
     }
 
@@ -604,8 +580,7 @@ compressed_to_ciphertext_new (gnutls_session_t session,
   ret =
     _gnutls_auth_cipher_encrypt2_tag (&params->write.cipher_state,
         cipher_data, length_to_encrypt, 
-        cipher_data, cipher_size,
-        tag_ptr, tag_size, compressed->size+2+pad);
+        cipher_data, cipher_size, 0);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -616,7 +591,7 @@ static void dummy_wait(record_parameters_st * params, gnutls_datum_t* plaintext,
                        unsigned pad_failed, unsigned int pad, unsigned total)
 {
   /* this hack is only needed on CBC ciphers */
-  if (_gnutls_cipher_is_block (params->cipher_algorithm) == CIPHER_BLOCK)
+  if (_gnutls_cipher_is_block (params->cipher) == CIPHER_BLOCK)
     {
       unsigned len;
 
@@ -625,7 +600,7 @@ static void dummy_wait(record_parameters_st * params, gnutls_datum_t* plaintext,
        */
       if (pad_failed == 0 && pad > 0) 
         {
-          len = _gnutls_get_hash_block_len(params->mac_algorithm);
+          len = _gnutls_mac_block_size(params->mac);
           if (len > 0)
             {
               /* This is really specific to the current hash functions.
@@ -654,6 +629,7 @@ ciphertext_to_compressed (gnutls_session_t session,
                           uint64* sequence)
 {
   uint8_t tag[MAX_HASH_SIZE];
+  const uint8_t* tag_ptr;
   unsigned int pad = 0, i;
   int length, length_to_decrypt;
   uint16_t blocksize;
@@ -662,17 +638,20 @@ ciphertext_to_compressed (gnutls_session_t session,
   unsigned int pad_failed = 0;
   uint8_t preamble[MAX_PREAMBLE_SIZE];
   unsigned int preamble_size;
-  unsigned int ver = gnutls_protocol_get_version (session);
+  const version_entry_st* ver = get_version (session);
   unsigned int tag_size = _gnutls_auth_cipher_tag_len (&params->read.cipher_state);
-  unsigned int explicit_iv = _gnutls_version_has_explicit_iv (session->security_parameters.version);
+  unsigned int explicit_iv = _gnutls_version_has_explicit_iv (ver);
   unsigned iv_size;
-  
-  iv_size = gnutls_cipher_get_iv_size(params->cipher_algorithm);
-  blocksize = gnutls_cipher_get_block_size (params->cipher_algorithm);
+
+  if (unlikely(ver == NULL))
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+  iv_size = _gnutls_cipher_get_iv_size(params->cipher);
+  blocksize = _gnutls_cipher_get_block_size (params->cipher);
 
   /* actual decryption (inplace)
    */
-  switch (_gnutls_cipher_is_block (params->cipher_algorithm))
+  switch (_gnutls_cipher_is_block (params->cipher))
     {
     case CIPHER_STREAM:
       /* The way AEAD ciphers are defined in RFC5246, it allows
@@ -683,10 +662,10 @@ ciphertext_to_compressed (gnutls_session_t session,
           uint8_t nonce[blocksize];
           /* Values in AEAD are pretty fixed in TLS 1.2 for 128-bit block
            */
-          if (params->read.IV.data == NULL || params->read.IV.size != 4)
+          if (unlikely(params->read.IV.data == NULL || params->read.IV.size != 4))
             return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
           
-          if (ciphertext->size < tag_size+AEAD_EXPLICIT_DATA_SIZE)
+          if (unlikely(ciphertext->size < tag_size+AEAD_EXPLICIT_DATA_SIZE))
             return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
           memcpy(nonce, params->read.IV.data, AEAD_IMPLICIT_DATA_SIZE);
@@ -697,22 +676,26 @@ ciphertext_to_compressed (gnutls_session_t session,
           ciphertext->data += AEAD_EXPLICIT_DATA_SIZE;
           ciphertext->size -= AEAD_EXPLICIT_DATA_SIZE;
           
-          length_to_decrypt = ciphertext->size - tag_size;
+          length = length_to_decrypt = ciphertext->size - tag_size;
+          tag_ptr = ciphertext->data + length_to_decrypt;
         }
       else if (iv_size > 0) 
         { /* a stream cipher with explicit IV */
           _gnutls_auth_cipher_setiv(&params->read.cipher_state, UINT64DATA(*sequence), 8);
           length_to_decrypt = ciphertext->size;
+          length = ciphertext->size - tag_size;
+          tag_ptr = compressed->data + length;
         }
       else
         {
-          if (ciphertext->size < tag_size)
+          if (unlikely(ciphertext->size < tag_size))
             return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
   
           length_to_decrypt = ciphertext->size;
+          length = ciphertext->size - tag_size;
+          tag_ptr = compressed->data + length;
         }
 
-      length = ciphertext->size - tag_size;
 
       /* Pass the type, version, length and compressed through
        * MAC.
@@ -723,18 +706,23 @@ ciphertext_to_compressed (gnutls_session_t session,
 
       _gnutls_auth_cipher_set_mac_nonce(&params->read.cipher_state, UINT64DATA(*sequence), 8);
       ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, preamble, preamble_size);
-      if (ret < 0)
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
 
-      if ((ret =
+      if (unlikely((unsigned)length_to_decrypt > compressed->size))
+        return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+      ret =
            _gnutls_auth_cipher_decrypt2 (&params->read.cipher_state,
              ciphertext->data, length_to_decrypt,
-             ciphertext->data, ciphertext->size)) < 0)
+             compressed->data, compressed->size);
+
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
 
       break;
     case CIPHER_BLOCK:
-      if (ciphertext->size < blocksize || (ciphertext->size % blocksize != 0))
+      if (unlikely(ciphertext->size < blocksize || (ciphertext->size % blocksize != 0)))
         return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
       /* ignore the IV in TLS 1.1+
@@ -748,7 +736,7 @@ ciphertext_to_compressed (gnutls_session_t session,
           ciphertext->data += blocksize;
         }
 
-      if (ciphertext->size < tag_size+1)
+      if (unlikely(ciphertext->size < tag_size+1))
         return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
 
       /* we don't use the auth_cipher interface here, since
@@ -756,25 +744,30 @@ ciphertext_to_compressed (gnutls_session_t session,
        * an API. (the length of plaintext is required to calculate
        * auth_data, but it is not available before decryption).
        */
-      if ((ret =
-           _gnutls_cipher_decrypt (&params->read.cipher_state.cipher,
-             ciphertext->data, ciphertext->size)) < 0)
+      if (unlikely(ciphertext->size > compressed->size))
+        return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
+
+      ret =
+           _gnutls_cipher_decrypt2 (&params->read.cipher_state.cipher,
+             ciphertext->data, ciphertext->size,
+             compressed->data, compressed->size);
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
 
-      pad = ciphertext->data[ciphertext->size - 1];   /* pad */
+      pad = compressed->data[ciphertext->size - 1];   /* pad */
 
       /* Check the pading bytes (TLS 1.x). 
        * Note that we access all 256 bytes of ciphertext for padding check
        * because there is a timing channel in that memory access (in certain CPUs).
        */
-      if (ver != GNUTLS_SSL3)
+      if (ver->id != GNUTLS_SSL3)
         for (i = 2; i <= MIN(256, ciphertext->size); i++)
           {
-            tmp_pad_failed |= (ciphertext->data[ciphertext->size - i] != pad);
+            tmp_pad_failed |= (compressed->data[ciphertext->size - i] != pad);
             pad_failed |= ((i<= (1+pad)) & (tmp_pad_failed));
           }
 
-      if (pad_failed != 0 || (1+pad > ((int) ciphertext->size - tag_size)))
+      if (unlikely(pad_failed != 0 || (1+pad > ((int) ciphertext->size - tag_size))))
         {
           /* We do not fail here. We check below for the
            * the pad_failed. If zero means success.
@@ -784,6 +777,7 @@ ciphertext_to_compressed (gnutls_session_t session,
         }
 
       length = ciphertext->size - tag_size - pad - 1;
+      tag_ptr = &compressed->data[length];
 
       /* Pass the type, version, length and compressed through
        * MAC.
@@ -794,11 +788,11 @@ ciphertext_to_compressed (gnutls_session_t session,
 
       _gnutls_auth_cipher_set_mac_nonce(&params->read.cipher_state, UINT64DATA(*sequence), 8);
       ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, preamble, preamble_size);
-      if (ret < 0)
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
 
-      ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, ciphertext->data, length);
-      if (ret < 0)
+      ret = _gnutls_auth_cipher_add_auth (&params->read.cipher_state, compressed->data, length);
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
 
       break;
@@ -807,10 +801,10 @@ ciphertext_to_compressed (gnutls_session_t session,
     }
 
   ret = _gnutls_auth_cipher_tag(&params->read.cipher_state, tag, tag_size);
-  if (ret < 0)
+  if (unlikely(ret < 0))
     return gnutls_assert_val(ret);
 
-  if (memcmp (tag, &ciphertext->data[length], tag_size) != 0 || pad_failed != 0)
+  if (unlikely(memcmp (tag, tag_ptr, tag_size) != 0 || pad_failed != 0))
     {
       /* HMAC was not the same. */
       dummy_wait(params, compressed, pad_failed, pad, length+preamble_size);
@@ -818,42 +812,38 @@ ciphertext_to_compressed (gnutls_session_t session,
       return gnutls_assert_val(GNUTLS_E_DECRYPTION_FAILED);
     }
 
-  /* copy the decrypted stuff to compressed_data.
-   */
-  if (compressed->size < (unsigned)length)
-    return gnutls_assert_val(GNUTLS_E_DECOMPRESSION_FAILED);
-
-  memcpy (compressed->data, ciphertext->data, length);
-
   return length;
 }
 
 static int
-ciphertext_to_compressed_new (gnutls_session_t session,
-                              gnutls_datum_t *ciphertext, 
-                              gnutls_datum_t * compressed,
-                              uint8_t type, record_parameters_st * params, 
-                              uint64* sequence)
+ciphertext_to_compressed_new (gnutls_session_t restrict session,
+                              gnutls_datum_t *restrict ciphertext, 
+                              gnutls_datum_t *restrict compressed,
+                              uint8_t type, record_parameters_st *restrict params, 
+                              uint64* restrict sequence)
 {
   uint8_t tag[MAX_HASH_SIZE];
-  uint8_t *tag_ptr;
+  const uint8_t *tag_ptr;
   unsigned int pad;
   int length, length_to_decrypt;
   uint16_t blocksize;
   int ret;
   uint8_t preamble[MAX_PREAMBLE_SIZE];
   unsigned int preamble_size;
-  unsigned int ver = gnutls_protocol_get_version (session);
+  const version_entry_st* ver = get_version (session);
   unsigned int tag_size = _gnutls_auth_cipher_tag_len (&params->read.cipher_state);
-  unsigned int explicit_iv = _gnutls_version_has_explicit_iv (session->security_parameters.version);
+  unsigned int explicit_iv = _gnutls_version_has_explicit_iv (ver);
   unsigned iv_size;
-  
-  iv_size = gnutls_cipher_get_iv_size(params->cipher_algorithm);
-  blocksize = gnutls_cipher_get_block_size (params->cipher_algorithm);
+
+  if (unlikely(ver == NULL))
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+  iv_size = _gnutls_cipher_get_iv_size(params->cipher);
+  blocksize = _gnutls_cipher_get_block_size (params->cipher);
 
   /* actual decryption (inplace)
    */
-  switch (_gnutls_cipher_is_block (params->cipher_algorithm))
+  switch (_gnutls_cipher_is_block (params->cipher))
     {
     case CIPHER_STREAM:
       /* The way AEAD ciphers are defined in RFC5246, it allows
