@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2009-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2009-2013 Free Software Foundation, Inc.
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -50,22 +51,20 @@ int _gnutls_cipher_exists(gnutls_cipher_algorithm_t cipher)
 }
 
 int
-_gnutls_cipher_init (cipher_hd_st * handle, gnutls_cipher_algorithm_t cipher,
+_gnutls_cipher_init (cipher_hd_st * handle, const cipher_entry_st* e,
                      const gnutls_datum_t * key, const gnutls_datum_t * iv, int enc)
 {
   int ret = GNUTLS_E_INTERNAL_ERROR;
   const gnutls_crypto_cipher_st *cc = NULL;
 
-  if (cipher == GNUTLS_CIPHER_NULL)
+  if (unlikely(e == NULL || e->id == GNUTLS_CIPHER_NULL))
     return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-  handle->is_aead = _gnutls_cipher_algo_is_aead(cipher);
-  if (handle->is_aead)
-     handle->tag_size = gnutls_cipher_get_block_size(cipher);
+  handle->e = e;
 
   /* check if a cipher has been registered
    */
-  cc = _gnutls_get_crypto_cipher (cipher);
+  cc = _gnutls_get_crypto_cipher (e->id);
   if (cc != NULL)
     {
       handle->encrypt = cc->encrypt;
@@ -75,7 +74,7 @@ _gnutls_cipher_init (cipher_hd_st * handle, gnutls_cipher_algorithm_t cipher,
       handle->tag = cc->tag;
       handle->setiv = cc->setiv;
 
-      SR (cc->init (cipher, &handle->handle, enc), cc_cleanup);
+      SR (cc->init (e->id, &handle->handle, enc), cc_cleanup);
       SR (cc->setkey( handle->handle, key->data, key->size), cc_cleanup);
       if (iv)
         {
@@ -94,7 +93,7 @@ _gnutls_cipher_init (cipher_hd_st * handle, gnutls_cipher_algorithm_t cipher,
 
   /* otherwise use generic cipher interface
    */
-  ret = _gnutls_cipher_ops.init (cipher, &handle->handle, enc);
+  ret = _gnutls_cipher_ops.init (e->id, &handle->handle, enc);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -131,49 +130,61 @@ cc_cleanup:
 /* Auth_cipher API 
  */
 int _gnutls_auth_cipher_init (auth_cipher_hd_st * handle, 
-  gnutls_cipher_algorithm_t cipher,
+  const cipher_entry_st* e,
   const gnutls_datum_t * cipher_key,
   const gnutls_datum_t * iv,
-  gnutls_mac_algorithm_t mac,
+  const mac_entry_st* me,
   const gnutls_datum_t * mac_key,
   int ssl_hmac, int enc)
 {
 int ret;
 
+  if (unlikely(e == NULL))
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
   memset(handle, 0, sizeof(*handle));
 
-  if (cipher != GNUTLS_CIPHER_NULL)
+  if (e->id != GNUTLS_CIPHER_NULL)
     {
-      ret = _gnutls_cipher_init(&handle->cipher, cipher, cipher_key, iv, enc);
+      handle->non_null = 1;
+      ret = _gnutls_cipher_init(&handle->cipher, e, cipher_key, iv, enc);
       if (ret < 0)
         return gnutls_assert_val(ret);
     }
   else
-    handle->is_null = 1;
+    handle->non_null = 0;
 
-  if (mac != GNUTLS_MAC_AEAD)
+  if (me->id != GNUTLS_MAC_AEAD)
     {
       handle->is_mac = 1;
       handle->ssl_hmac = ssl_hmac;
 
       if (ssl_hmac)
-        ret = _gnutls_mac_init_ssl3(&handle->mac.dig, mac, mac_key->data, mac_key->size);
+        ret = _gnutls_mac_init_ssl3(&handle->mac.dig, me, mac_key->data, mac_key->size);
       else
-        ret = _gnutls_mac_init(&handle->mac.mac, mac, mac_key->data, mac_key->size);
+        ret = _gnutls_mac_init(&handle->mac.mac, me, mac_key->data, mac_key->size);
       if (ret < 0)
         {
           gnutls_assert();
           goto cleanup;
         }
 
-      handle->tag_size = _gnutls_mac_get_algo_len(mac);
+      handle->tag_size = _gnutls_mac_get_algo_len(me);
+    }
+  else if (_gnutls_cipher_algo_is_aead(e))
+    {
+      handle->tag_size = _gnutls_cipher_get_tag_size(e);
+    }
+  else
+    {
+      gnutls_assert();
+      ret = GNUTLS_E_INVALID_REQUEST;
+      goto cleanup;
     }
-  else if (_gnutls_cipher_is_aead(&handle->cipher))
-    handle->tag_size = _gnutls_cipher_tag_len(&handle->cipher);
 
   return 0;
 cleanup:
-  if (handle->is_null == 0)
+  if (handle->non_null != 0)
     _gnutls_cipher_deinit(&handle->cipher);
   return ret;
 
@@ -195,45 +206,70 @@ int _gnutls_auth_cipher_add_auth (auth_cipher_hd_st * handle, const void *text,
     return 0;
 }
 
+/* The caller must make sure that textlen+pad_size+tag_size is divided by the block size of the cipher */
 int _gnutls_auth_cipher_encrypt2_tag (auth_cipher_hd_st * handle, const uint8_t *text,
-                      int textlen, void *ciphertext, int ciphertextlen, 
-                      void* tag_ptr, int tag_size,
-                      int auth_size)
+                      int textlen, void *_ciphertext, int ciphertextlen, 
+                      int pad_size)
 {
 int ret;
+uint8_t * ciphertext = _ciphertext;
+unsigned blocksize = _gnutls_cipher_get_block_size(handle->cipher.e);
+unsigned l;
 
   if (handle->is_mac)
     {
       if (handle->ssl_hmac)
-        ret = _gnutls_hash(&handle->mac.dig, text, auth_size);
+        ret = _gnutls_hash(&handle->mac.dig, text, textlen);
       else
-        ret = _gnutls_mac(&handle->mac.mac, text, auth_size);
-      if (ret < 0)
-        {
-          gnutls_assert();
-          return ret;
-        }
-      ret = _gnutls_auth_cipher_tag(handle, tag_ptr, tag_size);
+        ret = _gnutls_mac(&handle->mac.mac, text, textlen);
+      if (unlikely(ret < 0))
+        return gnutls_assert_val(ret);
+
+      if (unlikely(textlen+pad_size+handle->tag_size) > ciphertextlen)
+        return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+      ret = _gnutls_auth_cipher_tag(handle, ciphertext+textlen, handle->tag_size);
       if (ret < 0)
         return gnutls_assert_val(ret);
 
-      if (handle->is_null==0)
+      if (handle->non_null != 0)
         {
-          ret = _gnutls_cipher_encrypt2(&handle->cipher, text, textlen, ciphertext, ciphertextlen);
+          l = (textlen/blocksize)*blocksize;
+          ret = _gnutls_cipher_encrypt2(&handle->cipher, text, l, ciphertext, ciphertextlen);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
+
+          textlen -= l;
+          text += l;
+          ciphertext += l;
+          ciphertextlen -= l;
+
+          if (ciphertext != text && textlen > 0)
+            memcpy(ciphertext, text, textlen);
+
+          /* TLS 1.0 style padding */
+          if (pad_size > 0)
+            memset (ciphertext+textlen+handle->tag_size, pad_size - 1, pad_size);
+
+          ret = _gnutls_cipher_encrypt2(&handle->cipher, ciphertext, textlen+handle->tag_size+pad_size, ciphertext, ciphertextlen);
           if (ret < 0)
             return gnutls_assert_val(ret);
         }
+      else if (text != ciphertext)
+        memcpy(ciphertext, text, textlen);
     }
   else if (_gnutls_cipher_is_aead(&handle->cipher))
     {
       ret = _gnutls_cipher_encrypt2(&handle->cipher, text, textlen, ciphertext, ciphertextlen);
-      if (ret < 0)
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
 
-      ret = _gnutls_auth_cipher_tag(handle, tag_ptr, tag_size);
-      if (ret < 0)
+      ret = _gnutls_auth_cipher_tag(handle, ciphertext+textlen, handle->tag_size);
+      if (unlikely(ret < 0))
         return gnutls_assert_val(ret);
     }
+  else if (handle->non_null == 0 && text != ciphertext)
+    memcpy(ciphertext, text, textlen);
 
   return 0;
 }
@@ -244,23 +280,28 @@ int _gnutls_auth_cipher_decrypt2 (auth_cipher_hd_st * handle,
 {
 int ret;
 
-  if (handle->is_null==0)
+  if (unlikely(ciphertextlen > textlen))
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+  if (handle->non_null != 0)
     {
       ret = _gnutls_cipher_decrypt2(&handle->cipher, ciphertext, ciphertextlen, 
         text, textlen);
       if (ret < 0)
         return gnutls_assert_val(ret);
     }
+  else if (handle->non_null == 0 && text != ciphertext)
+    memcpy(text, ciphertext, ciphertextlen);
 
   if (handle->is_mac)
     {
       /* The MAC is not to be hashed */
-      textlen -= handle->tag_size;
+      ciphertextlen -= handle->tag_size;
 
       if (handle->ssl_hmac)
-        return _gnutls_hash(&handle->mac.dig, text, textlen);
+        return _gnutls_hash(&handle->mac.dig, text, ciphertextlen);
       else
-        return _gnutls_mac(&handle->mac.mac, text, textlen);
+        return _gnutls_mac(&handle->mac.mac, text, ciphertextlen);
     }
 
   return 0;
@@ -300,6 +341,6 @@ void _gnutls_auth_cipher_deinit (auth_cipher_hd_st * handle)
       else
         _gnutls_mac_deinit(&handle->mac.mac, NULL);
     }
-  if (handle->is_null==0)
+  if (handle->non_null!=0)
     _gnutls_cipher_deinit(&handle->cipher);
 }
