@@ -450,6 +450,46 @@ typedef struct record_state_st record_state_st;
 struct record_parameters_st;
 typedef struct record_parameters_st record_parameters_st;
 
+/* cipher and mac parameters */
+typedef struct cipher_entry_st
+{
+  const char *name;
+  gnutls_cipher_algorithm_t id;
+  uint16_t blocksize;
+  uint16_t keysize;
+  unsigned block:1;
+  uint16_t iv; /* the size of IV */
+  unsigned aead:1; /* Whether it is authenc cipher */
+} cipher_entry_st;
+
+typedef struct mac_entry_st
+{
+  const char *name;
+  const char *oid; /* OID of the hash - if it is a hash */
+  gnutls_mac_algorithm_t id;
+  unsigned output_size;
+  unsigned key_size;
+  unsigned nonce_size;
+  unsigned placeholder; /* if set, then not a real MAC */
+  unsigned secure; /* if set the this algorithm is secure as hash */
+  unsigned block_size; /* internal block size for HMAC */
+} mac_entry_st;
+
+typedef struct
+{
+  const char *name;
+  gnutls_protocol_t id;         /* gnutls internal version number */
+  uint8_t major;                    /* defined by the protocol */
+  uint8_t minor;                    /* defined by the protocol */
+  transport_t transport;	/* Type of transport, stream or datagram */
+  unsigned int supported:1;     /* 0 not supported, > 0 is supported */
+  unsigned int explicit_iv:1;
+  unsigned int extensions:1; /* whether it supports extensions */
+  unsigned int selectable_sighash:1; /* whether signatures can be selected */
+  unsigned int selectable_prf:1; /* whether the PRF is ciphersuite-defined */
+} version_entry_st;
+
+
 /* STATE (cont) */
 
 #include <gnutls_hash_int.h>
@@ -543,6 +583,7 @@ typedef struct
    * do that? Do they belong in security parameters?
    */
   int do_recv_supplemental, do_send_supplemental;
+  const version_entry_st* pversion;
 } security_parameters_st;
 
 struct record_state_st
@@ -555,6 +596,7 @@ struct record_state_st
   uint64 sequence_number;
 };
 
+
 /* These are used to resolve relative epochs. These values are just
    outside the 16 bit range to prevent off-by-one errors. An absolute
    epoch may be referred to by its numeric id in the range
@@ -568,10 +610,11 @@ struct record_parameters_st
   uint16_t epoch;
   int initialized;
 
-  gnutls_cipher_algorithm_t cipher_algorithm;
-  gnutls_mac_algorithm_t mac_algorithm;
   gnutls_compression_method_t compression_algorithm;
 
+  const cipher_entry_st* cipher;
+  const mac_entry_st* mac;
+
   /* for DTLS */
   uint64_t record_sw[DTLS_RECORD_WINDOW_SIZE];
   unsigned int record_sw_head_idx;
@@ -707,7 +750,6 @@ typedef struct
   unsigned int packets_dropped;
 } dtls_st;
 
-
 typedef union
 {
   void *ptr;
@@ -782,7 +824,7 @@ typedef struct
                                          * function.
                                          */
 
-  int expire_time;              /* after expire_time seconds this session will expire */
+  unsigned expire_time;              /* after expire_time seconds this session will expire */
   struct mod_auth_st_int *auth_struct;  /* used in handshake packets and KX algorithms */
 
   /* this is the highest version available
@@ -797,7 +839,7 @@ typedef struct
    * will be sent to the client. - only if the ciphersuite
    * supports it.
    */
-  int send_cert_req;
+  unsigned send_cert_req:1;
 
   size_t max_handshake_data_buffer_size;
 
@@ -825,15 +867,19 @@ typedef struct
   /* post client hello callback (server side only)
    */
   gnutls_handshake_post_client_hello_func user_hello_func;
+  /* handshake hook function */
+  gnutls_handshake_hook_func h_hook;
+  unsigned int h_type; /* the hooked type */
+  int16_t h_post; /* whether post-generation/receive */
 
   /* holds the selected certificate and key.
    * use _gnutls_selected_certs_deinit() and _gnutls_selected_certs_set()
    * to change them.
    */
   gnutls_pcert_st *selected_cert_list;
-  int selected_cert_list_length;
+  int16_t selected_cert_list_length;
   struct gnutls_privkey_st *selected_key;
-  int selected_need_free:1;
+  unsigned selected_need_free:1;
 
   /* holds the extensions we sent to the peer
    * (in case of a client)
@@ -844,7 +890,7 @@ typedef struct
   /* is 0 if we are to send the whole PGP key, or non zero
    * if the fingerprint is to be sent.
    */
-  int pgp_fingerprint;
+  unsigned pgp_fingerprint:1;
 
   /* This holds the default version that our first
    * record packet will have. */
@@ -852,7 +898,7 @@ typedef struct
 
   void *user_ptr;
 
-  int enable_private;           /* non zero to
+  unsigned enable_private:1;           /* non zero to
                                  * enable cipher suites
                                  * which have 0xFF status.
                                  */
@@ -860,7 +906,7 @@ typedef struct
   /* Holds 0 if the last called function was interrupted while
    * receiving, and non zero otherwise.
    */
-  int direction;
+  unsigned direction:1;
 
   /* This callback will be used (if set) to receive an
    * openpgp key. (if the peer sends a fingerprint)
@@ -870,7 +916,7 @@ typedef struct
   /* If non zero the server will not advertise the CA's he
    * trusts (do not send an RDN sequence).
    */
-  int ignore_rdn_sequence;
+  unsigned ignore_rdn_sequence:1;
 
   /* This is used to set an arbitary version in the RSA
    * PMS secret. Can be used by clients to test whether the
@@ -912,14 +958,14 @@ typedef struct
   {
     uint16_t type;
     extension_priv_data_t priv;
-    int set:1;
+    unsigned set:1;
   } extension_int_data[MAX_EXT_TYPES];
 
   struct
   {
     uint16_t type;
     extension_priv_data_t priv;
-    int set:1;
+    unsigned set:1;
   } resumed_extension_int_data[MAX_EXT_TYPES];
   /* The type of transport protocol; stream or datagram */
   transport_t transport;
@@ -946,7 +992,7 @@ typedef struct
   unsigned int hb_retrans_timeout_ms; /* the default timeout, in milliseconds*/
   unsigned int hb_total_timeout_ms; /* the total timeout, in milliseconds*/
   
-  unsigned int ocsp_check_ok; /* will be zero if the OCSP response TLS extension
+  unsigned int ocsp_check_ok:1; /* will be zero if the OCSP response TLS extension
                                * check failed (OCSP was old/unrelated or so). */
 
   heartbeat_state_t hb_state; /* for ping */
@@ -954,6 +1000,7 @@ typedef struct
   recv_state_t recv_state; /* state of the receive function */
   
   unsigned int sc_random_set:1;
+  unsigned int no_replay_protection:1; /* DTLS replay protection */
   
   /* If you add anything here, check _gnutls_handshake_internal_state_clear().
    */
@@ -988,25 +1035,21 @@ void _gnutls_free_auth_info (gnutls_session_t session);
 	session->internals.adv_version_major = major; \
 	session->internals.adv_version_minor = minor
 
-void _gnutls_set_adv_version (gnutls_session_t, gnutls_protocol_t);
-gnutls_protocol_t _gnutls_get_adv_version (gnutls_session_t);
-
 int _gnutls_is_secure_mem_null (const void *);
 
-inline static gnutls_protocol_t
-_gnutls_protocol_get_version (gnutls_session_t session)
+inline static const version_entry_st*
+get_version (gnutls_session_t session)
 {
-  return session->security_parameters.version;
+  return session->security_parameters.pversion;
 }
 
-#define gnutls_protocol_get_version _gnutls_protocol_get_version
+#define get_num_version(session) \
+	session->security_parameters.version
 
-inline static void
-_gnutls_set_current_version (gnutls_session_t session,
-                             gnutls_protocol_t version)
-{
-  session->security_parameters.version = version;
-}
+#define _gnutls_set_current_version(s, v) { \
+  s->security_parameters.version = v; \
+  s->security_parameters.pversion = version_to_entry(v); \
+  }
 
 #define timespec_sub_ms _gnutls_timespec_sub_ms
 unsigned int
