@@ -34,7 +34,6 @@
 #include "gnutls_mpi.h"
 #include <gnutls_state.h>
 #include <gnutls_datum.h>
-#include <gnutls_rsa_export.h>
 #include <gnutls_mbuffers.h>
 
 /* This is a temporary function to be used before the generate_*
@@ -111,7 +110,7 @@ generate_normal_master (gnutls_session_t session, gnutls_datum_t *premaster,
                                      security_parameters.server_random, 32,
                                      buf, sizeof (buf), NULL));
 
-  if (gnutls_protocol_get_version (session) == GNUTLS_SSL3)
+  if (get_num_version (session) == GNUTLS_SSL3)
     {
       uint8_t rnd[2 * GNUTLS_RANDOM_SIZE + 1];
 
@@ -368,7 +367,7 @@ _gnutls_send_client_certificate (gnutls_session_t session, int again)
 
   if (again == 0)
     {
-      if (gnutls_protocol_get_version (session) != GNUTLS_SSL3 ||
+      if (get_num_version (session) != GNUTLS_SSL3 ||
           session->internals.selected_cert_list_length > 0)
         {
           /* TLS 1.0 or SSL 3.0 with a valid certificate 
@@ -389,7 +388,7 @@ _gnutls_send_client_certificate (gnutls_session_t session, int again)
    * no certificate alert instead of an
    * empty certificate.
    */
-  if (gnutls_protocol_get_version (session) == GNUTLS_SSL3 &&
+  if (get_num_version (session) == GNUTLS_SSL3 &&
       session->internals.selected_cert_list_length == 0)
     {
       ret =
@@ -459,17 +458,6 @@ _gnutls_recv_server_kx_message (gnutls_session_t session)
 
   if (session->internals.auth_struct->gnutls_process_server_kx != NULL)
     {
-#ifdef ENABLE_RSA_EXPORT
-      /* EXCEPTION FOR RSA_EXPORT cipher suite 
-       */
-      if (_gnutls_session_is_export (session) != 0 &&
-          _gnutls_peers_cert_less_512 (session) != 0)
-        {
-          gnutls_assert ();
-          return 0;
-        }
-#endif
-
       /* Server key exchange packet is optional for PSK. */
       if (_gnutls_session_is_psk (session))
         optflag = 1;
@@ -602,7 +590,7 @@ _gnutls_recv_client_certificate (gnutls_session_t session)
        */
       if (optional != 0 &&
           ret == GNUTLS_E_WARNING_ALERT_RECEIVED &&
-          gnutls_protocol_get_version (session) == GNUTLS_SSL3 &&
+          get_num_version (session) == GNUTLS_SSL3 &&
           gnutls_alert_get (session) == GNUTLS_A_SSL3_NO_CERTIFICATE)
         {
 
