@@ -556,21 +556,17 @@ typedef void (rmadd_func) (priority_st * priority_list, unsigned int alg);
 static void
 prio_remove (priority_st * priority_list, unsigned int algo)
 {
-  int i = 0;
-  int pos = -1;                 /* the position of the cipher to remove */
+  unsigned int i;
 
-  while (priority_list->priority[i] != 0)
+  for (i=0;i<priority_list->algorithms;i++)
     {
-      if (priority_list->priority[i] == algo)
-        pos = i;
-      i++;
-    }
-
-  if (pos >= 0)
-    {
-      priority_list->priority[pos] = priority_list->priority[i - 1];
-      priority_list->priority[i - 1] = 0;
-      priority_list->algorithms--;
+      if (priority_list->priority[i] == algo) 
+        {
+          priority_list->algorithms--;
+          if ((priority_list->algorithms-i) > 0)
+            memmove(&priority_list->priority[i], &priority_list->priority[i+1], (priority_list->algorithms-i)*sizeof(priority_list->priority[0]));
+          break;
+        }
     }
 
   return;
@@ -816,7 +812,7 @@ bulk_rmadd_func *func;
  *
  * "NORMAL:-ARCFOUR-128" means normal ciphers except for ARCFOUR-128.
  *
- * "SECURE:-VERS-SSL3.0:+COMP-DEFLATE" means that only secure ciphers are
+ * "SECURE128:-VERS-SSL3.0:+COMP-DEFLATE" means that only secure ciphers are
  * enabled, SSL3.0 is disabled, and libz compression enabled.
  *
  * "NONE:+VERS-TLS-ALL:+AES-128-CBC:+RSA:+SHA1:+COMP-NULL:+SIGN-RSA-SHA1", 
