@@ -89,7 +89,7 @@ gnutls_cipher_get (gnutls_session_t session)
   if (ret < 0)
     return gnutls_assert_val(GNUTLS_CIPHER_NULL);
 
-  return record_params->cipher_algorithm;
+  return record_params->cipher->id;
 }
 
 /**
@@ -142,7 +142,7 @@ gnutls_mac_get (gnutls_session_t session)
   if (ret < 0)
     return gnutls_assert_val(GNUTLS_MAC_NULL);
 
-  return record_params->mac_algorithm;
+  return record_params->mac->id;
 }
 
 /**
@@ -292,6 +292,10 @@ _gnutls_handshake_internal_state_clear (gnutls_session_t session)
  * also available. The latter flag will enable a non-blocking
  * operation of the DTLS timers. 
  *
+ * The flag %GNUTLS_NO_REPLAY_PROTECTION will disable any 
+ * replay protection in DTLS mode. That must only used when 
+ * replay protection is achieved using other means.
+ *
  * Note that since version 3.1.2 this function enables some common
  * TLS extensions such as session tickets and OCSP certificate status
  * request in client side by default. To prevent that use the %GNUTLS_NO_EXTENSIONS
@@ -402,6 +406,9 @@ gnutls_init (gnutls_session_t * session, unsigned int flags)
       gnutls_ocsp_status_request_enable_client(*session, NULL, 0, NULL);
     }
 
+  if (flags & GNUTLS_NO_REPLAY_PROTECTION)
+    (*session)->internals.no_replay_protection = 1;
+
   return 0;
 }
 
@@ -479,7 +486,7 @@ gnutls_deinit (gnutls_session_t session)
   _gnutls_mpi_release (&session->key.rsa[1]);
 
   _gnutls_mpi_release (&session->key.dh_secret);
-
+  
   gnutls_free (session);
 }
 
@@ -746,13 +753,13 @@ gnutls_handshake_set_private_extensions (gnutls_session_t session, int allow)
 }
 
 inline static int
-_gnutls_cal_PRF_A (gnutls_mac_algorithm_t algorithm,
+_gnutls_cal_PRF_A (const mac_entry_st* me,
                    const void *secret, int secret_size,
                    const void *seed, int seed_size, void *result)
 {
   int ret;
 
-  ret = _gnutls_mac_fast (algorithm, secret, secret_size, seed, seed_size, result);
+  ret = _gnutls_mac_fast (me->id, secret, secret_size, seed, seed_size, result);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -775,6 +782,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
   int i, times, how, blocksize, A_size;
   uint8_t final[MAX_HASH_SIZE], Atmp[MAX_SEED_SIZE];
   int output_bytes, result;
+  const mac_entry_st* me = mac_to_entry(algorithm);
 
   if (seed_size > MAX_SEED_SIZE || total_bytes <= 0)
     {
@@ -782,7 +790,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
       return GNUTLS_E_INTERNAL_ERROR;
     }
 
-  blocksize = _gnutls_mac_get_algo_len (algorithm);
+  blocksize = _gnutls_mac_get_algo_len (me);
 
   output_bytes = 0;
   do
@@ -800,7 +808,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
 
   for (i = 0; i < times; i++)
     {
-      result = _gnutls_mac_init (&td2, algorithm, secret, secret_size);
+      result = _gnutls_mac_init (&td2, me, secret, secret_size);
       if (result < 0)
         {
           gnutls_assert ();
@@ -809,7 +817,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
 
       /* here we calculate A(i+1) */
       if ((result =
-           _gnutls_cal_PRF_A (algorithm, secret, secret_size, Atmp,
+           _gnutls_cal_PRF_A (me, secret, secret_size, Atmp,
                               A_size, Atmp)) < 0)
         {
           gnutls_assert ();
@@ -858,7 +866,7 @@ _gnutls_PRF (gnutls_session_t session,
   uint8_t s_seed[MAX_SEED_SIZE];
   uint8_t o1[MAX_PRF_BYTES], o2[MAX_PRF_BYTES];
   int result;
-  gnutls_protocol_t ver = gnutls_protocol_get_version (session);
+  const version_entry_st* ver = get_version (session);
 
   if (total_bytes > MAX_PRF_BYTES)
     {
@@ -986,7 +994,8 @@ gnutls_prf_raw (gnutls_session_t session,
  * @out: pre-allocated buffer to hold the generated data.
  *
  * Apply the TLS Pseudo-Random-Function (PRF) on the master secret
- * and the provided data, seeded with the client and server random fields.
+ * and the provided data, seeded with the client and server random fields,
+ * as specified in RFC5705.
  *
  * The @label variable usually contains a string denoting the purpose
  * for the generated data.  The @server_random_first indicates whether
@@ -1238,6 +1247,7 @@ gnutls_handshake_set_post_client_hello_function (gnutls_session_t session,
   session->internals.user_hello_func = func;
 }
 
+
 /**
  * gnutls_session_enable_compatibility_mode:
  * @session: is a #gnutls_session_t structure.
@@ -1249,6 +1259,9 @@ gnutls_handshake_set_post_client_hello_function (gnutls_session_t session,
  *
  * Normally only servers that require maximum compatibility with
  * everything out there, need to call this function.
+ *
+ * Note that this function must be called after any call to gnutls_priority
+ * functions.
  **/
 void
 gnutls_session_enable_compatibility_mode (gnutls_session_t session)
@@ -1310,7 +1323,6 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session)
   return _gnutls_session_ecc_curve_get(session);
 }
 
-#undef gnutls_protocol_get_version
 /**
  * gnutls_protocol_get_version:
  * @session: is a #gnutls_session_t structure.
@@ -1322,7 +1334,7 @@ gnutls_ecc_curve_t gnutls_ecc_curve_get(gnutls_session_t session)
 gnutls_protocol_t
 gnutls_protocol_get_version (gnutls_session_t session)
 {
-  return _gnutls_protocol_get_version(session);
+  return get_num_version(session);
 }
 
 /**
@@ -1396,3 +1408,41 @@ gnutls_handshake_set_random (gnutls_session_t session, const gnutls_datum_t* ran
   return 0;
 }
 
+/**
+ * gnutls_handshake_set_hook_function:
+ * @session: is a #gnutls_session_t structure
+ * @htype: the %gnutls_handshake_description_t of the message to hook at
+ * @post: %GNUTLS_HOOK_* depending on when the hook function should be called
+ * @func: is the function to be called
+ *
+ * This function will set a callback to be called after or before the specified
+ * handshake message has been received or generated. This is a
+ * generalization of gnutls_handshake_set_post_client_hello_function().
+ *
+ * To call the hook function prior to the message being sent/generation use 
+ * %GNUTLS_HOOK_PRE as @post parameter, %GNUTLS_HOOK_POST to call
+ * after, and %GNUTLS_HOOK_BOTH for both cases.
+ *
+ * This callback must return 0 on success or a gnutls error code to
+ * terminate the handshake.
+ *
+ * Note to hook at all handshake messages use an @htype of %GNUTLS_HANDSHAKE_ANY.
+ *
+ * Warning: You should not use this function to terminate the
+ * handshake based on client input unless you know what you are
+ * doing. Before the handshake is finished there is no way to know if
+ * there is a man-in-the-middle attack being performed.
+ *
+ * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
+ **/
+void
+gnutls_handshake_set_hook_function (gnutls_session_t session,
+                                    unsigned int htype,
+                                    int post,
+				    gnutls_handshake_hook_func func)
+{
+  session->internals.h_hook = func;
+  session->internals.h_type = htype;
+  session->internals.h_post = post;
+}
+
